@@ -20,12 +20,11 @@ pub(crate) type ApiResult<T> = Result<Output<T>, anyhow::Error>;
 pub(crate) struct Api {
     base: String,
     token: Option<SecretString>,
-    verbose: bool,
     user_agent: String,
 }
 
 impl Api {
-    pub(crate) fn new(management: &str, token: Option<&str>, verbose: bool) -> Self {
+    pub(crate) fn new(management: &str, token: Option<&str>) -> Self {
         let user_agent = format!("{}/{}", env!("CARGO_PKG_NAME"), env!("CARGO_PKG_VERSION"));
         let token = token.map(String::from).map(SecretString::new);
 
@@ -42,7 +41,6 @@ impl Api {
         Self {
             base,
             token,
-            verbose,
             user_agent,
         }
     }
@@ -229,9 +227,7 @@ impl Api {
     where
         T: fmt::Debug,
     {
-        if self.verbose {
-            println!("{}", output);
-        }
+        log::debug!("Output {}", output);
     }
 
     async fn del<P, T>(&self, path: P) -> ApiResult<T>
@@ -244,7 +240,7 @@ impl Api {
             .delete(url)
             .optionally_bearer_auth(self.token.as_ref())
             .inspect()
-            .send()
+            .retry()
             .await?
             .error_for_status2()
             .await?
@@ -262,7 +258,7 @@ impl Api {
             .get(url)
             .optionally_bearer_auth(self.token.as_ref())
             .inspect()
-            .send()
+            .retry()
             .await?
             .error_for_status2()
             .await?
@@ -284,8 +280,7 @@ impl Api {
             .optionally_bearer_auth(self.token.as_ref())
             .inspect()
             .optionally_json(body.as_ref())
-            // .json(&body)
-            .send()
+            .retry()
             .await?
             .error_for_status2()
             .await?
@@ -303,7 +298,7 @@ impl Api {
             .put(url)
             .optionally_bearer_auth(self.token.as_ref())
             .inspect()
-            .send()
+            .retry()
             .await?
             .error_for_status2()
             .await?
@@ -369,6 +364,29 @@ impl Inspector for reqwest::RequestBuilder {
     }
 }
 
+#[async_trait::async_trait]
+trait Retry {
+    async fn retry(self) -> reqwest::Result<reqwest::Response>;
+}
+
+#[async_trait::async_trait]
+impl Retry for reqwest::RequestBuilder {
+    async fn retry(self) -> reqwest::Result<reqwest::Response> {
+        loop {
+            if let Some(builder) = self.try_clone() {
+                let response = builder.send().await?;
+                if response.status().is_server_error() {
+                    log::trace!("Retrying on server error {}", response.status());
+                    continue;
+                }
+                break Ok(response);
+            } else {
+                break self.send().await;
+            }
+        }
+    }
+}
+
 #[async_trait::async_trait]
 trait ResponseExt: Sized {
     async fn split_response(self) -> reqwest::Result<(reqwest::StatusCode, bytes::Bytes)>;
@@ -379,11 +397,11 @@ trait ResponseExt: Sized {
 
         if status.is_server_error() {
             anyhow::bail!("Server error {}", status)
-        } else if status.is_client_error() {
-            anyhow::bail!("Client error {}", status)
-        } else {
-            Ok(bytes)
         }
+        if status.is_client_error() {
+            anyhow::bail!("Client error {}", status)
+        }
+        Ok(bytes)
     }
 }
 
